@@ -1,6 +1,7 @@
 import datetime
 import logging
 import os
+import re
 from http import HTTPStatus
 from typing import BinaryIO
 from typing import IO
@@ -17,6 +18,7 @@ from intezer_sdk import errors
 from intezer_sdk import operation
 from intezer_sdk._api import IntezerApi
 from intezer_sdk._util import deprecated
+from intezer_sdk.analyses_history import query_url_analyses_history
 from intezer_sdk.api import IntezerApiClient
 from intezer_sdk.api import get_global_api
 from intezer_sdk.base_analysis import Analysis
@@ -48,7 +50,8 @@ class FileAnalysis(Analysis):
                  code_item_type: str = None,
                  zip_password: str = None,
                  download_url: str = None,
-                 sandbox_command_line_arguments: str = None):
+                 sandbox_command_line_arguments: str = None,
+                 sandbox_machine_type: str = None):
         """
         FileAnalysis is a class for analyzing files. It is a subclass of the BaseAnalysis class and requires an API connection to Intezer.
 
@@ -63,6 +66,7 @@ class FileAnalysis(Analysis):
         :param zip_password: The password for a password-protected zip file.
         :param download_url: A URL from which to download the file to be analyzed.
         :param sandbox_command_line_arguments: The command line arguments for sandbox analysis.
+        :param sandbox_machine_type: The machine type to use in the sandbox. options are WIN7 or WIN10
         """
         super().__init__(api)
         if [file_path, file_hash, file_stream, download_url].count(None) < 3:
@@ -86,6 +90,7 @@ class FileAnalysis(Analysis):
         self._code_item_type = code_item_type
         self._zip_password = zip_password
         self._sandbox_command_line_arguments = sandbox_command_line_arguments
+        self._sandbox_machine_type = sandbox_machine_type
         self._sub_analyses: List[SubAnalysis] = None
         self._root_analysis = None
         self._iocs_report = None
@@ -120,6 +125,8 @@ class FileAnalysis(Analysis):
                                   file_hash: str,
                                   api: IntezerApiClient = None,
                                   private_only: bool = False,
+                                  composed_only: bool = None,
+                                  days_threshold_for_latest_analysis: int = None,
                                   **additional_parameters) -> Optional['FileAnalysis']:
         """
         Returns the latest FileAnalysis instance for the given file hash, with the option to filter by private analyses only.
@@ -128,10 +135,15 @@ class FileAnalysis(Analysis):
         :param file_hash: The hash of the file to retrieve analysis for.
         :param api: The API connection to Intezer.
         :param private_only: A flag to filter results by private analyses only.
+        :param composed_only: A flag to filter results by composed analyses only.
+        :param days_threshold_for_latest_analysis: The number of days to look back for the latest analysis.
         :param additional_parameters: Additional parameters to pass to the API.
         :return: The latest FileAnalysis instance for the given file hash.
         """
-        analysis_report = IntezerApi(api or get_global_api()).get_latest_analysis(file_hash, private_only, **additional_parameters)
+        analysis_report = IntezerApi(api or get_global_api()).get_latest_analysis(file_hash,
+                                                                                  private_only,
+                                                                                  composed_only,
+                                                                                  **additional_parameters)
 
         if not analysis_report:
             return None
@@ -139,6 +151,14 @@ class FileAnalysis(Analysis):
         analysis = cls(file_hash=file_hash, api=api)
         analysis._set_report(analysis_report)
 
+        if days_threshold_for_latest_analysis:
+            oldest_acceptable_date = datetime.datetime.utcnow() - datetime.timedelta(days=days_threshold_for_latest_analysis)
+
+            if analysis.analysis_time >= oldest_acceptable_date:
+                return analysis
+
+            return None
+
         return analysis
 
     def _query_status_from_api(self) -> Response:
@@ -153,6 +173,8 @@ class FileAnalysis(Analysis):
                                              self._disable_dynamic_unpacking,
                                              self._disable_static_unpacking,
                                              self._sandbox_command_line_arguments,
+                                             self._sandbox_machine_type,
+                                             self._file_name,
                                              **additional_parameters)
         elif self._download_url:
             return self._api.analyze_by_download_url(
@@ -162,9 +184,10 @@ class FileAnalysis(Analysis):
                 code_item_type=self._code_item_type,
                 zip_password=self._zip_password,
                 sandbox_command_line_arguments=self._sandbox_command_line_arguments,
+                sandbox_machine_type=self._sandbox_machine_type,
                 **additional_parameters)
         else:
-            return self._api.analyze_by_file(self._file_path,
+            analysis_id = self._api.analyze_by_file(self._file_path,
                                              self._file_stream,
                                              disable_dynamic_unpacking=self._disable_dynamic_unpacking,
                                              disable_static_unpacking=self._disable_static_unpacking,
@@ -172,7 +195,10 @@ class FileAnalysis(Analysis):
                                              code_item_type=self._code_item_type,
                                              zip_password=self._zip_password,
                                              sandbox_command_line_arguments=self._sandbox_command_line_arguments,
+                                             sandbox_machine_type=self._sandbox_machine_type,
                                              **additional_parameters)
+            self._file_stream = None
+            return analysis_id
 
     def get_sub_analyses(self) -> List[SubAnalysis]:
         """
@@ -209,15 +235,16 @@ class FileAnalysis(Analysis):
             else:
                 self._sub_analyses.append(sub_analysis_object)
 
-    def download_file(self, path: str = None, output_stream: IO = None):
+    def download_file(self, path: str = None, output_stream: IO = None, password_protection: str = None):
         """
         Downloads the analysis's file.
 
         ``path`` or ``output_stream`` must be provided.
         :param path: A path to where to save the file, it can be either a directory or non-existing file path.
         :param output_stream: A file-like object to write the file's content to.
+        :param password_protection: set password protection to download file as zip with password.
         """
-        self._api.download_file_by_sha256(self.result()['sha256'], path, output_stream)
+        self._api.download_file_by_sha256(self.result()['sha256'], path, output_stream, password_protection)
 
     @property
     def iocs(self) -> dict:
@@ -276,6 +303,22 @@ class FileAnalysis(Analysis):
 
         return self._dynamic_ttps_report
 
+    @property
+    def verdict(self) -> str:
+        """
+        The analysis verdict.
+        """
+        self._assert_analysis_finished()
+        return self._report['verdict']
+
+    @property
+    def sub_verdict(self) -> str:
+        """
+        The analysis sub-verdict.
+        """
+        self._assert_analysis_finished()
+        return self._report['sub_verdict']
+
 
 @deprecated('This method is deprecated, use FileAnalysis.from_latest_hash_analysis instead to be explict')
 def get_latest_analysis(file_hash: str,
@@ -294,6 +337,15 @@ def get_file_analysis_by_id(analysis_id: str, api: IntezerApi = None) -> Optiona
 def get_analysis_by_id(analysis_id: str, api: IntezerApi = None) -> Optional[FileAnalysis]:
     return get_file_analysis_by_id(analysis_id, api)
 
+def _clean_url(url: str) -> str:
+    """
+    Remove http:// or https:// or www. from the beginning of the URL,
+    and / from the end of the URL.
+    """
+    url = re.sub(r'^https?://(www\.)?', '', url)
+    url = re.sub(r'\/$', '', url)
+
+    return url
 
 class UrlAnalysis(Analysis):
     """
@@ -308,6 +360,7 @@ class UrlAnalysis(Analysis):
     :ivar url: The analyzed url
     :vartype url: str
     """
+
     def __init__(self, url: Optional[str] = None, api: IntezerApiClient = None):
         """
          UrlAnalysis is a class for analyzing URLs.
@@ -316,7 +369,7 @@ class UrlAnalysis(Analysis):
         :param api: The API connection to Intezer.
         """
         super().__init__(api)
-        self._api.assert_any_on_premise()
+        self._api.assert_any_on_premise('query-url-history')
         self.url = url
         self._file_analysis: Optional[FileAnalysis] = None
 
@@ -333,6 +386,54 @@ class UrlAnalysis(Analysis):
         response = IntezerApi(api or get_global_api()).get_url_analysis_response(analysis_id, True)
         return cls._create_analysis_from_response(response, api, analysis_id)
 
+    @classmethod
+    def from_latest_analysis(cls,
+                             url: str,
+                             days_threshold_for_latest_analysis: int = 1,
+                             api: IntezerApiClient = None,
+                             exact_match: bool = False) -> Optional['UrlAnalysis']:
+        """
+        Returns a UrlAnalysis instance with the latest analysis of the given URL.
+        Note: For more control over the query (beyond the submitted URL), use the 'query_url_analyses_history' method.
+        :param url: The 'submitted URL' to retrieve the latest analysis for.
+        :param days_threshold_for_latest_analysis: The number of days to look back for the latest analysis.
+        :param api: The API connection to Intezer.
+        :param exact_match: If True, the URL must match exactly. Otherwise, try to find similar URLs which were analyzed.
+        :return: A UrlAnalysis instance with the latest analysis of the given URL.
+        """
+        now = datetime.datetime.now()
+        yesterday = now - datetime.timedelta(days=days_threshold_for_latest_analysis)
+
+        url = url if exact_match else _clean_url(url)
+        analysis_history_url_result = query_url_analyses_history(start_date=yesterday,
+                                                                 end_date=now,
+                                                                 submitted_url=url,
+                                                                 aggregated_view=True,
+                                                                 api=api,
+                                                                 exact_match=exact_match)
+        analyses_ids = [report['analysis_id'] for report in analysis_history_url_result.all()]
+
+        if not analyses_ids:
+            return None
+
+        return cls.from_analysis_id(analyses_ids[0], api=api)
+
+    @property
+    def verdict(self) -> str:
+        """
+        The analysis verdict.
+        """
+        self._assert_analysis_finished()
+        return self._report['summary']['verdict_type']
+
+    @property
+    def sub_verdict(self) -> str:
+        """
+        The analysis sub-verdict.
+        """
+        self._assert_analysis_finished()
+        return self._report['summary']['verdict_name']
+
     def _set_report(self, report: dict):
         super()._set_report(report)
         if not self.url:
@@ -359,6 +460,9 @@ class UrlAnalysis(Analysis):
         if 'downloaded_file' not in self._report:
             return None
 
+        if not self._report['downloaded_file'].get('analysis_id'):
+            return None
+
         file_analysis_id = self._report['downloaded_file']['analysis_id']
         self._file_analysis = FileAnalysis.from_analysis_id(file_analysis_id, self._api.api)
         return self._file_analysis
